@@ -6,10 +6,13 @@
 # http://www.popbill.com
 # Author : Kim Hyunjin (code@linkhubcorp.com)
 # Written : 2018-02-26
-# Updated : 2018-08-09
+# Updated : 2022-08-03
 # Thanks for your interest.
 from .base import PopbillBase, PopbillException, File
-
+try:
+    from urllib import parse as parse
+except ImportError:
+    import urllib as parse
 
 class KakaoService(PopbillBase):
     """ 팝빌 카카오톡 API Service Implementation. """
@@ -29,7 +32,7 @@ class KakaoService(PopbillBase):
         """
         :param CorpNum: 팝빌회원 사업자번호
         :param UserID: 팝빌회원 아이디
-        :param ToGo: [PLUSFRIEND-플러스친구계정관리, SENDER-발신번호관리, TEMPLATE-알림톡템플릿관리, BOX-카카오톡전송내용]
+        :param ToGo: [PLUSFRIEND-카카오톡채널관리, SENDER-발신번호관리, TEMPLATE-알림톡템플릿관리, BOX-카카오톡전송내용]
         :return: 팝빌 URL
         """
         if ToGo == None or ToGo == '':
@@ -43,7 +46,7 @@ class KakaoService(PopbillBase):
 
     def getPlusFriendMgtURL(self, CorpNum, UserID):
         """
-        플러스친구 계정관리 팝업 URL
+        카카오톡 채널 관리 팝업 URL
         :param CorpNum: 팝빌회원 사업자번호
         :param UserID: 팝빌회원 아이디
         :return: 팝빌 URL
@@ -83,13 +86,29 @@ class KakaoService(PopbillBase):
 
     def listPlusFriendID(self, CorpNum, UserID=None):
         """
-        플러스친구 목록 확인
+        카카오톡 채널 목록 확인
         :param CorpNum: 팝빌회원 사업자번호
         :param UserID: 팝빌회원 아이디
-        :return: 플러스친구 list
+        :return: 카카오톡 채널 list
         """
         return self._httpget('/KakaoTalk/ListPlusFriendID', CorpNum, UserID)
 
+    def checkSenderNumber(self, CorpNum, SenderNumber, UserID=None):
+        """ 발신번호 등록여부 확인
+            args
+                CorpNum : 회원 사업자번호
+                SenderNumber : 확인할 발신번호
+                UserID  : 회원 팝빌아이디
+            return
+                처리결과. consist of code and message
+            raise
+                PopbillException
+        """
+        if SenderNumber == None or SenderNumber == '':
+            raise PopbillException(-99999999, "발신번호가 입력되지 않았습니다.")
+
+        return self._httpget('/KakaoTalk/CheckSenderNumber/' + SenderNumber, CorpNum, UserID)
+
     def getSenderNumberList(self, CorpNum, UserID=None):
         """
         발신번호 목록 확인
@@ -105,7 +124,8 @@ class KakaoService(PopbillBase):
         :param CorpNum: 팝빌회원 사업자번호
         :param templateCode: 알림톡 템플릿 코드
         :param UserID: 팝빌회원 아이디
-        :return: templateCode (템플릿 코드), templateName (템플릿 이름), template (템플릿 내용), plusFriendID (채널 이름)
+        :return: templateCode (템플릿 코드), templateName (템플릿 이름), template (템플릿 내용), plusFriendID (채널 이름),
+                 secureYN (보안템플릿 여부),  state (템플릿 상태), stateDT (템플릿 상태 일시)
         """
         if templateCode == None or templateCode == '':
             raise PopbillException(-99999999, "템플릿 코드가 입력되지 않았습니다.")
@@ -135,6 +155,7 @@ class KakaoService(PopbillBase):
         return self.sendATS_same(CorpNum, TemplateCode, Sender, "", "", AltSendType, SndDT, KakaoMessages, UserID,
                                  RequestNum)
 
+    # 버튼 추가
     def sendATS(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, Receiver,
                 ReceiverName, UserID=None, RequestNum=None, ButtonList=None):
 
@@ -148,6 +169,21 @@ class KakaoService(PopbillBase):
         return self.sendATS_same(CorpNum, TemplateCode, Sender, "", "", AltSendType, SndDT, KakaoMessages, UserID,
                                  RequestNum, ButtonList)
 
+    # 대체문자 제목 추가
+    def sendATS(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, Receiver,
+                ReceiverName, UserID=None, RequestNum=None, ButtonList=None, AltSubject=None):
+
+        KakaoMessages = []
+        KakaoMessages.append(KakaoReceiver(
+            rcv=Receiver,
+            rcvnm=ReceiverName,
+            msg=Content,
+            altsjt=AltSubject,
+            altmsg=AltContent)
+        )
+        return self.sendATS_same(CorpNum, TemplateCode, Sender, "", "", AltSendType, SndDT, KakaoMessages, UserID,
+                                 RequestNum, ButtonList, "")
+
     def sendATS_multi(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
                       UserID=None,
                       RequestNum=None):
@@ -159,13 +195,26 @@ class KakaoService(PopbillBase):
         return self.sendATS_same(CorpNum, TemplateCode, Sender, "", "", AltSendType, SndDT, KakaoMessages, UserID,
                                  RequestNum, ButtonList)
 
+    # 대체문자 제목 추가
+    def sendATS_multi(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
+                      UserID=None, RequestNum=None, ButtonList=None, AltSubject=None):
+        return self.sendATS_same(CorpNum, TemplateCode, Sender, "", "", AltSendType, SndDT, KakaoMessages, UserID,
+                                 RequestNum, ButtonList, AltSubject)
+
     def sendATS_same(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
                      UserID=None, RequestNum=None):
         return self.sendATS_same(CorpNum,TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
-                    UserID, RequestNum, None)
+                    UserID, RequestNum, None, None)
+
     # 버튼 추가
     def sendATS_same(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
                      UserID=None, RequestNum=None, ButtonList=None):
+        return self.sendATS_same(CorpNum,TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
+                    UserID, RequestNum, None, None)
+
+    # 대체문자 제목 추가
+    def sendATS_same(self, CorpNum, TemplateCode, Sender, Content, AltContent, AltSendType, SndDT, KakaoMessages,
+                     UserID=None, RequestNum=None, ButtonList=None, AltSubject=None):
         """
        알림톡 대량 전송
        :param CorpNum: 팝빌회원 사업자번호
@@ -178,6 +227,8 @@ class KakaoService(PopbillBase):
        :param KakaoMessages: 알림톡 내용 (배열)
        :param UserID: 팝빌회원 아이디
        :param RequestNum : 요청번호
+       :param ButtonList : 버튼 (배열)
+       :param AltSubject : 대체문자 제목
        :return: receiptNum (접수번호)
        """
         if TemplateCode is None or TemplateCode == '':
@@ -193,6 +244,8 @@ class KakaoService(PopbillBase):
             req['snd'] = Sender
         if Content is not None or Content != '':
             req['content'] = Content
+        if AltSubject is not None or AltSubject != '':
+            req['altSubject'] = AltSubject
         if AltContent is not None or AltContent != '':
             req['altContent'] = AltContent
         if AltSendType is not None or AltSendType != '':
@@ -225,17 +278,44 @@ class KakaoService(PopbillBase):
         return self.sendFTS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, KakaoMessages, KakaoButtons,
                                  AdsYN, UserID, RequestNum)
 
+    # 대체문자 제목 추가
+    def sendFTS(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, Receiver,
+                ReceiverName, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
+        KakaoMessages = []
+        KakaoMessages.append(KakaoReceiver(
+            rcv=Receiver,
+            rcvnm=ReceiverName,
+            msg=Content,
+            altsjt= AltSubject,
+            altmsg=AltContent)
+        )
+
+        return self.sendFTS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, KakaoMessages, KakaoButtons,
+                                 AdsYN, UserID, RequestNum, "")
+
     def sendFTS_multi(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT,
                       KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None):
         return self.sendFTS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, KakaoMessages, KakaoButtons,
                                  AdsYN, UserID, RequestNum)
 
+    # 대체문자 제목 추가
+    def sendFTS_multi(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT,
+                      KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
+        return self.sendFTS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, KakaoMessages, KakaoButtons,
+                                 AdsYN, UserID, RequestNum, AltSubject)
+
     def sendFTS_same(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT,
                      KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None):
+        return self.sendFTS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, KakaoMessages, KakaoButtons,
+                                 AdsYN, UserID, RequestNum, None)
+
+    # 대체문자 제목 추가
+    def sendFTS_same(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT,
+                     KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
         """
         친구톡 텍스트 대량 전송
         :param CorpNum: 팝빌회원 사업자번호
-        :param PlusFriendID: 플러스친구 아이디
+        :param PlusFriendID: 검색용 아이디
         :param Sender: 발신번호
         :param Content: [동보] 친구톡 내용
         :param AltContent: [동보] 대체문자 내용
@@ -246,10 +326,11 @@ class KakaoService(PopbillBase):
         :param AdsYN: 광고 전송여부
         :param UserID: 팝빌회원 아이디
         :param RequestNum : 요청번호
+        :param AltSubject : 대체문자 제목
         :return: receiptNum (접수번호)
         """
         if PlusFriendID is None or PlusFriendID == '':
-            raise PopbillException(-99999999, "플러스친구 아이디가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "검색용 아이디가 입력되지 않았습니다.")
         if Sender is None or Sender == '':
             raise PopbillException(-99999999, "발신번호가 입력되지 않았습니다.")
 
@@ -262,6 +343,8 @@ class KakaoService(PopbillBase):
             req['altSendType'] = AltSendType
         if Content is not None or Content != '':
             req['content'] = Content
+        if AltSubject is not None or AltSubject != '':
+            req['altSubject'] = AltSubject
         if AltContent is not None or AltContent != '':
             req['altContent'] = AltContent
         if SndDT is not None or SndDT != '':
@@ -295,17 +378,45 @@ class KakaoService(PopbillBase):
         return self.sendFMS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, FilePath, ImageURL,
                                  KakaoMessages, KakaoButtons, AdsYN, UserID, RequestNum)
 
+    # 대체문자 제목 추가
+    def sendFMS(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, FilePath, ImageURL,
+                Receiver, ReceiverName, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
+
+        KakaoMessages = []
+        KakaoMessages.append(KakaoReceiver(
+            rcv=Receiver,
+            rcvnm=ReceiverName,
+            msg=Content,
+            altsjt=AltSubject,
+            altmsg=AltContent)
+        )
+
+        return self.sendFMS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, FilePath, ImageURL,
+                                 KakaoMessages, KakaoButtons, AdsYN, UserID, RequestNum, "")
+
     def sendFMS_multi(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, FilePath, ImageURL,
                       KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None):
         return self.sendFMS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, FilePath, ImageURL,
                                  KakaoMessages, KakaoButtons, AdsYN, UserID, RequestNum)
 
+    # 대체문자 제목 추가
+    def sendFMS_multi(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, FilePath, ImageURL,
+                      KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
+        return self.sendFMS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, FilePath, ImageURL,
+                                 KakaoMessages, KakaoButtons, AdsYN, UserID, RequestNum, None)
+
     def sendFMS_same(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, FilePath, ImageURL,
                      KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None):
+        return self.sendFMS_same(CorpNum, PlusFriendID, Sender, "", "", AltSendType, SndDT, FilePath, ImageURL,
+                                 KakaoMessages, KakaoButtons, AdsYN, UserID, RequestNum, None)
+
+    # 대체문자 제목 추가
+    def sendFMS_same(self, CorpNum, PlusFriendID, Sender, Content, AltContent, AltSendType, SndDT, FilePath, ImageURL,
+                     KakaoMessages, KakaoButtons, AdsYN=False, UserID=None, RequestNum=None, AltSubject=None):
         """
         친구톡 이미지 대량 전송
         :param CorpNum: 팝빌회원 사업자번호
-        :param PlusFriendID: 플러스친구 아이디
+        :param PlusFriendID: 검색용 아이디
         :param Sender: 발신번호
         :param Content: [동보] 친구톡 내용
         :param AltContent: [동보] 대체문자 내용
@@ -318,10 +429,11 @@ class KakaoService(PopbillBase):
         :param AdsYN: 광고 전송여부
         :param UserID: 팝빌회원 아이디
         :param RequestNum : 요청번호
+        :param AltSubject : 대체문자 제목
         :return: receiptNum (접수번호)
         """
         if PlusFriendID is None or PlusFriendID == '':
-            raise PopbillException(-99999999, "플러스친구 아이디가 입력되지 않았습니다.")
+            raise PopbillException(-99999999, "검색용 아이디가 입력되지 않았습니다.")
         if Sender is None or Sender == '':
             raise PopbillException(-99999999, "발신번호가 입력되지 않았습니다.")
 
@@ -332,6 +444,8 @@ class KakaoService(PopbillBase):
             req['snd'] = Sender
         if Content is not None or Content != '':
             req['content'] = Content
+        if AltSubject is not None or AltSubject != '':
+            req['altSubject'] = AltSubject
         if AltContent is not None or AltContent != '':
             req['altContent'] = AltContent
         if AltSendType is not None or AltSendType != '':
@@ -416,7 +530,7 @@ class KakaoService(PopbillBase):
 
         return self._httpget('/KakaoTalk/Get/' + RequestNum, CorpNum, UserID)
 
-    def search(self, CorpNum, SDate, EDate, State, Item, ReserveYN, SenderYN, Page, PerPage, Order, UserID,
+    def search(self, CorpNum, SDate, EDate, State, Item, ReserveYN, SenderYN, Page, PerPage, Order, UserID=None,
                QString=None):
 
         """
@@ -455,7 +569,7 @@ class KakaoService(PopbillBase):
         uri += '&Order=' + Order
 
         if QString is not None:
-            uri += '&QString=' + QString
+            uri += '&QString=' + parse.quote(QString)
 
         return self._httpget(uri, CorpNum, UserID)
 
@@ -486,7 +600,7 @@ class KakaoService(PopbillBase):
 
 class KakaoReceiver(object):
     def __init__(self, **kwargs):
-        self.__dict__ = dict.fromkeys(['rcv', 'rcvnm', 'msg', 'altmsg'])
+        self.__dict__ = dict.fromkeys(['rcv', 'rcvnm', 'msg', 'altsjt', 'altmsg', 'btns', 'interOPRefKey'])
         self.__dict__.update(kwargs)
 
 
